<?php
/**
 * 文件型数据库 SQLite 3 引挈，该引挈仅支持 PHP5.3 及更高版本
 * @作者 qinggan <admin@phpok.com>
 * @版权 深圳市锟铻科技有限公司
 * @主页 http://www.phpok.com
 * @版本 4.x
 * @许可 http://www.phpok.com/lgpl.html PHPOK开源授权协议：GNU Lesser General Public License
 * @时间 2017年09月08日
**/

/**
 * 安全限制，防止直接访问
**/
if(!defined("PHPOK_SET")){
	exit("<h1>Access Denied</h1>");
}

class db_sqlite extends db
{
	private $type = SQLITE3_ASSOC;
	
	public function __construct($config=array())
	{
		parent::__construct($config);
		$this->config($config);
		$this->kec('[',']');
	}

	public function __destruct()
	{
		$this->close();
	}

	public function config($config)
	{
		parent::config($config);
	}

	/**
	 * 连接SQLite数据库
	**/
	public function connect()
	{
		if(!$this->database){
			$this->error('数据库文件未设置');
		}
		if(!file_exists($this->database)){
			$this->error('数据库文件不存在');
		}
		$this->_time();
		$this->conn = new SQLite3($this->database,SQLITE3_OPEN_READWRITE);
		if(!$this->conn){
			$this->error('数据库连接失败');
		}
		$this->conn->busyTimeout(100);
		$this->_time();
		$this->query("PRAGMA encoding = 'UTF-8'");
		return $this->conn;
	}

	public function close()
	{
		if($this->conn && is_object($this->conn)){
			$this->conn->close();
		}
	}

	public function type($type='')
	{
		if($type && ($type == 'num' || $type == SQLITE3_NUM)){
			$this->type = SQLITE3_NUM;
		}else{
			$this->type = SQLITE3_ASSOC;
		}
		return $this->type;
	}

	public function set($name,$value)
	{
		if($name == "rs_type" || $name == 'type'){
			$value = strtolower($value) == "num" ? SQLITE3_NUM : SQLITE3_ASSOC;
			$this->type = $value;
		}else{
			$this->$name = $value;
		}
	}

	/**
	 * 检测链接是否存在
	**/
	private function check_connect()
	{
		if(!$this->conn || !is_object($this->conn)){
			$this->connect();
		}
	}

	public function query($sql,$loadcache=true)
	{
		if($loadcache){
			$this->cache_sql($sql);
		}
		$this->check_connect();
		$this->_time();
		$this->query = $this->conn->query($sql);
		if($loadcache){
			$this->cache_update($sql);
		}
		$tmptime = $this->_time();
		$this->_count();
		if($this->debug){
			$this->debug($sql,$tmptime);
		}
		if($errid = $this->conn->lastErrorCode()){
			$this->error($this->conn->lastErrorMsg(),$errid);
		}
		return $this->query;
	}

	public function get_all($sql,$primary="")
	{
		$this->query($sql);
		if(!$this->query || !is_object($this->query)){
			return false;
		}
		$this->_time();
		$rs = false;
		while($rows = $this->query->fetchArray($this->type)){
			if($primary){
				$rs[$rows[$primary]] = $rows;
			}else{
				$rs[] = $rows;
			}
		}
		$this->query->finalize();
		if($rs){
			$rs = $this->decode($rs);
		}
		$this->_time();
		return $rs;
	}

	public function get_one($sql="")
	{
		$this->query($sql);
		if(!$this->query || !is_object($this->query)){
			return false;
		}
		$this->_time();
		$rs = $this->query->fetchArray($this->type);
		$this->query->finalize();
		if($rs){
			$rs = $this->decode($rs);
		}
		$this->_time();
		return $rs;
	}

	public function insert_id()
	{
		$this->check_connect();
		return $this->conn->lastInsertRowID();
	}

	/**
	 * 查看有多少数量
	 * @参数 $sql 要查询的SQL
	 * @参数 $is_count 是否使用 SQL 自带的 count 
	 * @参数 
	**/
	public function count($sql="",$is_count=true)
	{
		if($sql && $is_count){
			$this->set('type','num');
			$rs = $this->get_one($sql);
			$this->set('type','assoc');
			return $rs[0];
		}else{
			if($sql){
				$this->query($sql);
			}
			if($this->query){
				$this->query->reset();
				$i = 0;
				while($rows = $this->query->fetch_array($this->type)){
					$i++;
				}
				return $i;
			}
			return false;
		}
	}

	/**
	 * 查询字段个数
	 * @参数 $sql 要查询的语句
	**/
	public function num_fields($sql="")
	{
		if($sql){
			$this->query($sql);
		}
		if($this->query){
			return $this->query->numColumns();
		}
		return false;
	}

	public function list_fields($table)
	{
		$rslist = $this->list_fields_more($table);
		if(!$rslist){
			return false;
		}
		$tmplist = array();
		foreach($rslist as $key=>$value){
			$tmplist[] = $value['name'];
		}
		return $tmplist;
	}

	public function list_fields_more($table)
	{
		if(substr($table,0,strlen($this->prefix)) != $this->prefix){
			$table = $this->prefix.$table;
		}
		$sql = "PRAGMA table_info(".$table.")";
		return $this->get_all($sql);
	}

	public function list_tables()
	{
		$rslist = $this->get_all("SELECT tbl_name FROM sqlite_master WHERE type='table'");
		if(!$rslist){
			return false;
		}
		$rs = array();
		foreach($rslist as $key=>$value){
			$rs[] = $value["tbl_name"];
		}
		return $rs;
	}

	//显示表名
	public function table_name($table_list,$i)
	{
		return $table_list[$i];
	}

	public function phpok_one($tbl,$condition="",$fields="*")
	{
		if(substr($table,0,strlen($this->prefix)) != $this->prefix){
			$table = $this->prefix.$table;
		}
		$sql = "SELECT ".$fields." FROM ".$table;
		if($condition){
			$sql .= " WHERE ".$condition;
		}
		return $this->get_one($sql);
	}

	/**
	 * 取得版本号
	**/
	public function version($type="server")
	{
		return $this->conn->version();
	}

	public function decode($char)
	{
		if(!$char){
			return false;
		}
		if(is_array($char)){
			foreach($char as $key=>$value){
				if($value){
					$char[$key] = $this->decode($value);
				}
			}
		}else{
			$char = str_replace("\'\'", "'", $char);
			$char = str_replace('\"', '"', $char);
		}
		return $char;
	}

	public function escape_string($char)
	{
		return $this->encode($char);
	}

	public function encode($char)
	{
		if(!$char){
			return false;
		}
		if(is_array($char)){
			foreach($char as $key=>$value){
				if($value){
					$char[$key] = $this->encode($value);
				}
			}
		}else{
			$char = $this->conn->escapeString(stripslashes($char));
		}
		return $char;
	}

	/**
	 * 删除字段
	**/
	public function field_delete($table,$name)
	{
		if(!$table || !$name){
			return false;
		}
		if(!is_array($name)){
			$name = explode(",",$name);
		}
		if(substr($table,0,strlen($this->prefix)) != $this->prefix){
			$table = $this->prefix.$table;
		}
		$rslist = $this->list_fields_more($table);
		if(!$rslist){
			return true;
		}
		//检测字段是否存在，不存在就跳过重建
		$state = false;
		foreach($rslist as $key=>$value){
			if($name && in_array($value['name'],$name)){
				$state = true;
				break;
			}
		}
		if(!$state){
			return true;
		}
		$tmpid = 0;
		$data = array();
		foreach($rslist as $key=>$value){
			if($name && in_array($value['name'],$name)){
				continue;
			}
			$value['cid'] = $tmpid;
			$data[] = $value;
			$tmpid++;
		}
		$this->_field_update($table,$data);
		return true;
	}

	/**
	 * 字段属性更新
	**/
	public function field_update($table,$name,$data = array())
	{
		if(!$table || !$name){
			return false;
		}
		if(substr($table,0,strlen($this->prefix)) != $this->prefix){
			$table = $this->prefix.$table;
		}
		$rslist = $this->list_fields_more($table);
		if(!$rslist){
			return true;
		}
		foreach($rslist as $key=>$value){
			if($value['name'] == $name){
				if($data && $data['dflt_value'] != ''){
					$value['dflt_value'] = $data['dflt_value'];
				}
				if($data && $data['type'] != ''){
					$value['type'] = $data['type'];
				}
				if($data && $data['notnull']){
					$value['notnull'] = $data['notnull'];
				}
				$rslist[$key] = $value;
			}
		}
		return $this->_field_update($table,$rslist);
	}

	private function _field_update($table,$rslist)
	{
		$sql = "ALTER TABLE ".$table." RENAME TO ".$table."_tmp";
		$this->conn->query($sql);
		$sql = "CREATE TABLE ".$table." (";
		foreach($rslist as $key=>$value){
			if($key>0){
				$sql .= ",";
			}
			$sql .= "[".$value['name']."] ";
			if($value['pk']){
				if($value['type'] == 'integer'){
					$sql .= "INTEGER PRIMARY KEY AUTOINCREMENT";
				} else {
					$sql .= $value['type']." PRIMARY KEY ";
				}
			}else{
				$sql .= " ".$value['type']." ";
			}
			if($value['dflt_value']){
				$sql .= $value['notnull'] ? ' NOT NULL ' : ' NULL ';
				$sql .= " DEFAULT ".(is_numeric($value['dflt_value']) ? $value['dflt_value'] : "'".$value['dflt_value']."'");
			}else{
				$sql .= " NULL ";
			}
		}
		$sql .= ")";
		$this->conn->query($sql);
		//迁移数据
		$sql = "INSERT INTO ".$table."(";
		foreach($rslist as $key=>$value){
			if($value['cid'] >0){
				$sql .= ",";
			}
			$sql .= $value['name'];
		}
		$sql .= ") SELECT ";
		foreach($rslist as $key=>$value){
			if($value['cid'] >0){
				$sql .= ",";
			}
			$sql .= $value['name'];
		}
		$sql .= " FROM ".$table."_tmp";
		$this->conn->query($sql);
		$this->conn->query("DROP TABLE ".$table."_tmp");
		return true;
	}

	public function field_create($table,$name,$data=array())
	{
		if(!$table || !$name){
			return false;
		}
		if(substr($table,0,strlen($this->prefix)) != $this->prefix){
			$table = $this->prefix.$table;
		}
		$sql = "ALTER TABLE ".$table." ADD COLUMN [".$name."]";
		if($data && $data['pk']){
			if($value['type'] == 'integer'){
				$sql .= "INTEGER PRIMARY KEY AUTOINCREMENT";
			} else {
				$sql .= $value['type']." PRIMARY KEY ";
			}
		}else{
			$sql .= " ".$data['type']." ";
		}
		if($data && $data['dflt_value']){
			$sql .= $data['notnull'] ? ' NOT NULL ' : ' NULL ';
			$sql .= " DEFAULT ".(is_numeric($data['dflt_value']) ? $data['dflt_value'] : "'".$data['dflt_value']."'");
		}else{
			$sql .= " NULL ";
		}
		return $this->query($sql);
	}
}